<?php


namespace RUMCWV;

if (!defined('ABSPATH')) {
	die('Let\'s not do this');
}


class RUMCWV_RumDB
{

	public static $aMetrics  = ['LCP','FID','CLS','FCP','INP','TTFB'];
	public static $aScores = [
		'CLS' => [0.1, 0.25],
		'FCP' => [1800, 3000],
		'LCP' => [2500, 4000],
		'FID' => [100, 300],
		'INP' => [200, 500],
		'TTFB' => [800, 1800],
	];

	public static function createTable()
	{

		global $wpdb;
		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

		foreach (['rum_core_web_vitals', 'rum_cwv_tmp'] as $tablename) {

			$table_name_rum_cwv_rum = $wpdb->prefix . $tablename;

			$sql = "CREATE TABLE $table_name_rum_cwv_rum ( 
				`id` INT(11) NOT NULL AUTO_INCREMENT ,
				`LCP` FLOAT(10) DEFAULT NULL,
				`FID` FLOAT(10) DEFAULT NULL,
				`CLS` FLOAT(10) DEFAULT NULL,
				`FCP` FLOAT(10) DEFAULT NULL,
				`INP` FLOAT(10) DEFAULT NULL,
				`TTFB` FLOAT(10) DEFAULT NULL,
				`u` VARCHAR(255) DEFAULT NULL,
				`d` ENUM('desktop', 'mobile') DEFAULT 'desktop',
				`date` DATETIME DEFAULT CURRENT_TIMESTAMP,
				PRIMARY KEY (`id`)
			);";

			dbDelta($sql);
		}

		self::demodata();
	}


	public static function droptable(){
		global $wpdb;

		foreach (['rum_core_web_vitals', 'rum_cwv_tmp'] as $tablename) {
			$table_name_rum_cwv_rum = $wpdb->prefix . $tablename;
			$wpdb->query("DROP TABLE IF EXISTS $table_name_rum_cwv_rum");
		}

	}

	

	/**
	 * truncate all rum tables
	 */
	public static function truncate()
	{
		global $wpdb;
		foreach (['rum_core_web_vitals', 'rum_cwv_tmp'] as $tablename) {
			$table_name_rum_cwv_rum = $wpdb->prefix . $tablename;
			$wpdb->query("TRUNCATE TABLE $table_name_rum_cwv_rum");
		}
	}

	/**
	 * Demonstrate the RUM functionality
	 **/
	public static function demodata()
	{
		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_core_web_vitals';
		$wpdb->query("INSERT INTO $table_name_rum_cwv_rum (LCP, FID, CLS, FCP, INP, TTFB, u, d) VALUES (600, 10, 0.02, 800, 40, 400, '/', 'desktop')");
		$wpdb->query("INSERT INTO $table_name_rum_cwv_rum (LCP, FID, CLS, FCP, INP, TTFB, u, d) VALUES (1200, 12, 0.04, 900, 44, 420, '/', 'desktop')");
		$wpdb->query("INSERT INTO $table_name_rum_cwv_rum (LCP, FID, CLS, FCP, INP, TTFB, u, d) VALUES (1400, 14, 0.06, 1000, 48, 440, '/', 'mobile')");
		$wpdb->query("INSERT INTO $table_name_rum_cwv_rum (LCP, FID, CLS, FCP, INP, TTFB, u, d) VALUES (1400, 14, 0.06, 1000, 48, 440, '/', 'mobile')");
	}

	/**
	 * move the tmp table to the main table
	 */
	public static function tmp_to_main()
	{

		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_core_web_vitals';
		$table_name_rum_cwv_rum_tmp = $wpdb->prefix . 'rum_cwv_tmp';
		$wpdb->query("INSERT INTO $table_name_rum_cwv_rum (LCP, FID, CLS, FCP, INP, TTFB, u, d, date) SELECT LCP, FID, CLS, FCP, INP, TTFB, u, d, date FROM $table_name_rum_cwv_rum_tmp");
		$wpdb->query("TRUNCATE TABLE $table_name_rum_cwv_rum_tmp");
	}

	


	public static function deleteOlder()
	{

		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_core_web_vitals';
		$wpdb->query("DELETE FROM  $table_name_rum_cwv_rum WHERE date < DATE_SUB(NOW(), INTERVAL 30 DAY)");
	}


	/**
	 * inserts or updates the rum data into the tmp table
	 * @todo: do one big insert for first data batch
	 * @todo: consider using a temporary table and wp-cron this to the main table
	 * @param $id
	 * @param $metricname
	 * @param $metricvalue
	 * @param $url
	 * @param $device
	 */

	public static function upsert($id, $metricname, $metricvalue, $url, $device)
	{
		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_cwv_tmp';

		// try to insert
		$sql = "UPDATE $table_name_rum_cwv_rum SET $metricname = '%f', u = '%s', d = '%s' WHERE id = %d";
		$sql = $wpdb->prepare($sql, array($metricvalue, $url, $device, $id));
		$test = $wpdb->query($sql);

		if ($test <> 1) {
			$sql = "INSERT INTO $table_name_rum_cwv_rum (id,$metricname,u,d) VALUES (%d,%f,%s,%s)";
			$sql = $wpdb->prepare($sql, array($id, $metricvalue, $url, $device));
			$wpdb->query($sql);
		}
	}

	/**
	 * returns the the good-improve-bad distribution for a metric
	 * @param $metric
	 * @return array
	 */
	public static function progress($metric,$device)
	{
		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_core_web_vitals';

		$where = in_array($device,["desktop","mobile"]) ? " AND d = '" . $device . "'" : "";

		$sql = "
			SELECT progressbar , count(*) AS Count
			FROM (
				SELECT CASE WHEN $metric BETWEEN -1 AND " . self::$aScores[$metric][0] . " THEN 'good'
							WHEN $metric BETWEEN " . self::$aScores[$metric][0] . " AND " . self::$aScores[$metric][1] . " THEN 'improve'
							ELSE 'bad'
						END as progressbar
				FROM $table_name_rum_cwv_rum 
				WHERE $metric IS NOT NULL $where
			) progressbar 
			group by progressbar
		";

		$rows =  $wpdb->get_results($sql);
		return (is_array($rows)) ? $rows : [];
	}

	/**
	 * returns the the histogram for a metric
	 * @param $metric
	 * @return array
	 */
	public static function histogram($metric,$device)
	{
		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_core_web_vitals';

		$where = in_array($device,["desktop","mobile"]) ? " AND d = '" . $device . "'" : "";


		$max = self::percentile($metric, 95);
		$bucketstep = round($max / 16);
		if ($bucketstep < 0.01) {
			$bucketstep = '0.01';
		}


		$sql = "
			SELECT 
				ROUND($metric / $bucketstep) * $bucketstep as histogram, 
				count(*) as Count
			FROM $table_name_rum_cwv_rum
			WHERE $metric IS NOT NULL $where
			GROUP BY histogram
			ORDER BY histogram ASC
		";

		$rows =  $wpdb->get_results($sql);
		$rows = (is_array($rows)) ? $rows : [];

		return [
			'bucketstep' => $bucketstep,
			'steps' => 16,
			'rows' => $rows
		];
	}

	public static function timeline($metric, $device,$p = 75)
	{
		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_core_web_vitals';

		$where = in_array($device,["desktop","mobile"]) ? " AND d = '" . $device . "'" : "";

		$sql = "SELECT DATE(date) as date,
				SUBSTRING_INDEX(SUBSTRING_INDEX( GROUP_CONCAT($metric ORDER BY $metric SEPARATOR ','), ',', $p/100 * COUNT(*) ), ',', -1)
				AS p75 
				FROM $table_name_rum_cwv_rum WHERE $metric IS NOT NULL $where GROUP BY DATE(date) ORDER BY date ASC";

		$rows = $wpdb->get_results($sql);

		return (is_array($rows)) ? $rows : [];
	}


	public static function urllist($metric, $device,$p = 75)
	{
		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_core_web_vitals';

		$where = in_array($device,["desktop","mobile"]) ? " AND d = '" . $device . "'" : "";

		$sql = "SELECT u as u,
				CAST(
					SUBSTRING_INDEX(SUBSTRING_INDEX( GROUP_CONCAT($metric ORDER BY $metric SEPARATOR ','), ',', $p/100 * COUNT(*) ), ',', -1)
				 AS DECIMAL(10,2))
				AS p75,  
				COUNT(*) as count
				FROM $table_name_rum_cwv_rum WHERE $metric IS NOT NULL $where GROUP BY u ORDER BY count DESC";

		$rows = $wpdb->get_results($sql);

		return (is_array($rows)) ? $rows : [];
	}



	/**
	 * returns the 75th percentile
	 * @param $metric
	 * @return float|int
	 */
	public static function percentile($metric,$device, $p = 75)
	{
		global $wpdb;
		$table_name_rum_cwv_rum = $wpdb->prefix . 'rum_core_web_vitals';

		$where = in_array($device,["desktop","mobile"]) ? " AND d = '" . $device . "'" : "";

		$sql = "SELECT 
				SUBSTRING_INDEX(SUBSTRING_INDEX( GROUP_CONCAT($metric ORDER BY $metric SEPARATOR ','), ',', $p/100 * COUNT(*) ), ',', -1)
				AS 95thPer 
				FROM $table_name_rum_cwv_rum WHERE $metric IS NOT NULL $where";

		$c = $wpdb->get_var($sql);

		return (float)$c;
	}


	
}
